import fs from 'fs';
import Headers from './headers.node';
import { readFile, createReadStream } from './read-file.node';

const isDataURL = url => url.startsWith('data:');

const isRequestURL = url => url.startsWith('http:') || url.startsWith('https:');

export default class NodeFetchResponse {
  constructor(url, options) {
    this._url = url;
    this._ok = true;
    this._status = 200;
    this.options = options;
    this.bodyUsed = false;
    this._headers = null;
  }

  get ok() {
    return this._ok;
  }

  get status() {
    return this._status;
  }

  get url() {
    return this._url;
  }

  get headers() {
    this._headers = this._headers || this._getHeaders();
    return this._headers;
  }

  get body() {
    const {
      url,
      options
    } = this;
    this.bodyUsed = true;
    return createReadStream(url, options);
  }

  async arrayBuffer() {
    this.bodyUsed = true;
    const data = await readFile(this.url, this.options);
    return data;
  }

  async text() {
    const arrayBuffer = await this.arrayBuffer();
    const textDecoder = new TextDecoder();
    return textDecoder.decode(arrayBuffer);
  }

  async json() {
    const text = await this.text();
    return JSON.parse(text);
  }

  _getHeaders() {
    const headers = {};

    if (this.httpResponse) {
      const httpHeaders = this.httpResponse.getHeaders();

      for (const name in httpHeaders) {
        const header = headers[name];
        headers[name] = String(header);
      }
    } else {
      const contentLength = this._getContentLength();

      if (Number.isFinite(contentLength)) {
        headers['Content-Length'] = contentLength;
      }
    }

    return new Headers(headers);
  }

  _getContentLength() {
    if (isRequestURL(this.url)) {
      return null;
    } else if (isDataURL(this.url)) {
      return this.url.length - 'data:'.length;
    }

    const stats = fs.statSync(this.url);
    return stats.size;
  }

}
//# sourceMappingURL=response.node.js.map